package com.diagbot.facade;

import com.diagbot.client.AuthServiceClient;
import com.diagbot.dto.LoginDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.JWT;
import com.diagbot.entity.User;
import com.diagbot.exception.CommonException;
import com.diagbot.exception.ErrorCode;
import com.diagbot.service.impl.UserServiceImpl;
import com.diagbot.util.UserUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.security.crypto.factory.PasswordEncoderFactories;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.util.Date;

/**
 * @Description: 用户业务层
 * @author: gaodm
 * @time: 2018/8/6 9:00
 */
@Component
public class UserFacade extends UserServiceImpl {

    @Autowired
    AuthServiceClient authServiceClient;

    public User createUser(User user){
        PasswordEncoder passwordEncoder = PasswordEncoderFactories.createDelegatingPasswordEncoder();
        String entryPassword= passwordEncoder.encode(user.getPassword());
        user.setPassword(entryPassword);
        this.save(user);
        return  user;
    }

    @Cacheable(value = "UserName", key = "'username:'+#p0")
    public User getUserInfo(String username){
        return this.findByName(username);
    }

    public RespDTO login(String username , String password){
        User user= this.findByName(username);
        if(null==user){
            throw new CommonException(ErrorCode.USER_NOT_FOUND);
        }
        PasswordEncoder passwordEncoder = PasswordEncoderFactories.createDelegatingPasswordEncoder();
        if(!passwordEncoder.matches(password,user.getPassword())){
            throw new CommonException(ErrorCode.USER_PASSWORD_ERROR);
        }

        JWT jwt = authServiceClient.getToken("Basic dWFhLXNlcnZpY2U6MTIzNDU2", "password", username, password);
        // 获得用户菜单
        if(null==jwt){
            throw new CommonException(ErrorCode.GET_TOKEN_FAIL);
        }
        LoginDTO loginDTO=new LoginDTO();
        loginDTO.setUser(user);
        loginDTO.setToken(jwt.getAccess_token());
        return RespDTO.onSuc(loginDTO);
    }

    private static final String CACHE_NAME = "UserInfo";

    //缓存演示
    @CachePut(value = CACHE_NAME, key = "'user:'+#p0['id']")
    public User saveUser(User user){
        PasswordEncoder passwordEncoder = PasswordEncoderFactories.createDelegatingPasswordEncoder();
        String entryPassword= passwordEncoder.encode(user.getPassword());
        user.setPassword(entryPassword);
        user.setGmtCreate(new Date());
        user.setCreator(UserUtils.getCurrentPrincipleID());
        super.save(user);
        return user;
    }

    @CachePut(value = CACHE_NAME, key = "'user:'+#p0['id']")
    public User updateUser(User user){
        PasswordEncoder passwordEncoder = PasswordEncoderFactories.createDelegatingPasswordEncoder();
        String entryPassword= passwordEncoder.encode(user.getPassword());
        user.setPassword(entryPassword);
        user.setGmtModified(new Date());
        user.setModifier(UserUtils.getCurrentPrincipleID());
        super.updateById(user);
        return user;
    }

    @Override
    @Cacheable(value = CACHE_NAME, key = "'user:'+#p0")
    public User getById(Serializable id){
        return super.getById(id);
    }

    @Override
    @CacheEvict(value = CACHE_NAME, key = "'user:'+#p0")
    public boolean removeById(Serializable id){
        return super.removeById(id);
    }


}
